/* Sample program using autoSql routines to load and save
 * an address book to a tab-delimited file. */
#include "common.h"
#include "linefile.h"
#include "addressBook.h"

char *fileName = "addressBook.txt";	/* Our address book file. */
struct addressBook *addressList = NULL;	/* Our address book in memory. */

void usage()
/* Print usage info and exit. */
{
errAbort(
 "address - save or find an address in address book\n"
 "usage:\n"
 "   address find \"name\"n"
 "This will print out info for someone of given name\n"
 "   address add \"name\" \"address\" \"city\" \"state\" zipCode\n"
 "This will add to the address book\n"
 "   address delete \"name\"n"
 "Remove someone from address book.\n"
 );
}

void saveAddressBook()
/* Save address book. */
{
FILE *f = mustOpen(fileName, "w");
struct addressBook *ab;
for (ab = addressList; ab != NULL; ab = ab->next)
    addressBookTabOut(ab, f);
fclose(f);
}

void loadAddressBook()
/* Load address book. */
{
struct lineFile *lf = lineFileMayOpen(fileName, TRUE);
char *line, *words[16];
struct addressBook *ab;

while (lineFileNext(lf, &line, NULL))
    {
    chopTabs(line, words);
    ab = addressBookLoad(words);
    slAddHead(&addressList, ab);
    }
slReverse(&addressList); /* Add at head/reverse faster than add at tail. */
}

struct addressBook *findAddress(char *name)
/* Search through address book for name. */
{
struct addressBook *ab;
for (ab = addressList; ab != NULL; ab = ab->next)
    if (sameString(name, ab->name))
        return ab;
errAbort("Couldn't find %s in %s\n", name, fileName);
}

void find(char *name)
/* Find name in address book and print out info. */
{
struct addressBook *ab  = findAddress(name);
printf("%s\n", ab->name);
printf("%s\n", ab->address);
printf("%s, %s %d\n", ab->city, ab->state, ab->zipCode);
}

void delete(char *name)
/* Delete someone from address book. */
{
struct addressBook *ab  = findAddress(name);
slRemoveEl(&addressList, ab);
addressBookFree(&ab);
saveAddressBook();
}

void add(char *name, char *address, char *city, char *state, char *zipCode)
/* Add new record to address book. */
{
struct addressBook *ab;
AllocVar(ab);
ab->name = cloneString(name);
ab->address = cloneString(address);
ab->city = cloneString(city);
ab->zipCode = atoi(zipCode);
strncpy(ab->state, state, sizeof(ab->state));
slAddTail(&addressList, ab);
saveAddressBook();
}

int main(int argc, char *argv[])
/* Parse command line and print usage info. Load up address book at
 * start and free it when we're done. */
{
char *command;

/* Load existing address book if it exists. */
loadAddressBook();

/* Parse command line and dispatch to appropriate routine. */
if (argc < 3)
    usage();
command = argv[1];
if (sameWord(command, "find"))
     find(argv[2]);
else if (sameWord(command, "delete"))
     delete(argv[2]);
else if (sameWord(command, "add"))
     {
     if (argc != 7)
         usage();
     add(argv[2], argv[3], argv[4], argv[5], argv[6]);
     }

/* Free up memory version of address book. */
addressBookFreeList(&addressList);	
return 0;
}
